--------------------------------------------------------------------------------------
-- Route Terminator Script - can be used on any route
-- Used for the end of a line, where we want the preceding signal(s) to show a warning aspect
-- RSDL / Rail Simulator
--------------------------------------------------------------------------------------

-- Included for constants only
--include=New Common Signal Script.lua

--------------------------------------------------------------------------------------
-- INITIALISE
--
function Initialise ()

	-- Tells the game to do an update tick once the route finishes loading
	-- This will ensure the other signals are loaded before the route terminator sends a blocked message to them
	Call( "BeginUpdate" )
end

--------------------------------------------------------------------------------------
-- UPDATE
--
function Update ( time )

	-- Send a Blocked message to set the preceding home signal to warning
	Call( "SendSignalMessage", SIGNAL_BLOCKED, "EndOfRoute", -1, 1, 0 )

	-- Set the signal to appear as blocked on the 2D map
	Call( "Set2DMapSignalState", BLOCKED )

	Call( "EndUpdate" )
end

--------------------------------------------------------------------------------------
-- ON CONSIST PASS
-- Called when a train passes one of the signal's links
--
function OnConsistPass ( prevFrontDist, prevBackDist, frontDist, backDist, linkIndex )
	-- Do nothing!
end

-------------------------------------------------------------------------------------
-- ON SIGNAL MESSAGE
-- Handles messages from other signals.
--
function OnSignalMessage( message, parameter, direction, linkIndex )

	-- SIGNAL RESET

	if (message == RESET_SIGNAL_STATE) then

		-- Resets signal state, if scenario is reset
		ResetSignalState()
	end

	-- Otherwise DO NOTHING!
end

--------------------------------------------------------------------------------------
-- GET SIGNAL STATE
-- Gets the current state of the signal - blocked, warning or clear
--
function GetSignalState( )

	-- We're always BLOCKED!
	return BLOCKED
end

-------------------------------------------------------------------------------
--                                                                           --
-- Signals\CommonScripts                                                     --
--          N e w   C o m m o n   S i g n a l  S c r i p t . l u a           --
--                                                                           --
-------------------------------------------------------------------------------

-- GLOBALS
DIR_UP = -1
DIR_DOWN = 1

-- States
CLEAR = 0
WARNING = 1
BLOCKED = 2

-- KJM 09-Nov-2009 Texture swap constants
PRIMARY_TEXT = 0
SECONDARY_TEXT = 1

-- Signal Messages (0-9 are reserved by code)
RESET_SIGNAL_STATE								= 0
INITIALISE_SIGNAL_TO_BLOCKED 					= 1
JUNCTION_STATE_CHANGE							= 2
INITIALISE_TO_PREPARED							= 3
REQUEST_TO_PASS_DANGER							= 4

-- Locally defined signal mesages
OCCUPATION_INCREMENT							= 10
OCCUPATION_DECREMENT							= 11

SIGNAL_BLOCKED									= 12
SIGNAL_CLEARED									= 13
SIGNAL_WARNING									= 14
SIGNAL_WARNING2									= 15

-- special signal messages only used by the signals that message the opposite facing signal
-- in the section of track where two single direction tracks converge to one dual direction track
	-- currently only used for UK Semaphore signals
OCCUPATION_REVERSE_INCREMENT 		 			= 16
OCCUPATION_REVERSE_DECREMENT 					= 17

-- UK / GERMAN SPECIFIC functionality: used to set preparedness of control signals (which default to red)
SIGNAL_UNPREPARED								= 18
SIGNAL_PREPARED									= 19
SIGNAL_PREPARED_VISIBLE							= 20	-- Special state for UK signals with approach control
SIGNAL_PREPARED_ALWAYS							= 20	-- Special state for German yard exit signals

-- GERMAN SPECIFIC functionality: used to let signals behind us know speed limit and which route we're connected to (eg, for Zs2/3)
SIGNAL_ROUTE_INDICATION							= 21
SIGNAL_ROUTE_SPEED_LIMIT						= 22

-- Signal Messages 33-39 are available for later extension
-- Signal Messages 40-49 reserved for end user expansion


-- What you need to add to a signal message number to turn it into the equivalent PASS message
PASS_OFFSET										= 50


-- Pass on messages to handle overlapping links (eg for converging junctions / crossovers)
	-- Populated by BaseInitialise()
PASS = { }


-- SPAD and warning system messages to pass to consist
AWS_MESSAGE										= 11
TPWS_MESSAGE									= 12
SPAD_MESSAGE 									= 14


-- Script global variables
gUpdating		= 0								-- are we updating?
gConnectedLink	= 0								-- which link is connected?



-- debugging stuff
DEBUG = false 									-- set to true to turn debugging on
function DebugPrint( message )
	if (DEBUG) then
		Print( message )
	end
end

--------------------------------------------------------------------------------------
-- BASE INITIALISE
-- initialise function used by all signal scripts
--
function BaseInitialise()

	-- Initialise PASS constants
	for i = 0, PASS_OFFSET do
		PASS[i] = i + PASS_OFFSET
	end

	-- Initialise global variables
	gLinkCount = Call( "GetLinkCount" )			-- number of links this signal has
	gId = Call( "GetId" )						-- name of this signal
	if ( gId == "" ) then
		gId = "NA"
	end
	-- Initialise tables
	gLinkState = {}								-- state of line beyond this link
	gYardEntry = {}								-- is this link going inside a yard?
	gOccupationTable = {}						-- how many trains are in this part of our block?

	for link = 0, gLinkCount - 1 do
		gLinkState[link] = SIGNAL_CLEARED
		gYardEntry[link] = false
		gOccupationTable[link] = 0
	end
end

--------------------------------------------------------------------------------------
-- RESET SIGNAL STATE
-- Resets the signal when the route / scenario is reloaded
--
function ResetSignalState ( )

	DebugPrint( "DEBUG: ResetSignalState() started")

	-- Re-initialise the signal
	Initialise()

	DebugPrint( "DEBUG: ResetSignalState() ended")

end

